﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Security;
using System.Runtime.Serialization;
using System.IO;
using System.Data.SqlClient;
using System.Configuration;
using System.Web.UI.WebControls;

namespace ePlanner
{
    public enum ConnectionStateEventType
    {
        Closed,
        Open,
        Lost,
        Restore
    }

    public delegate void ConnectionStateChangedEventHandler(object sender, ConnectionStateEventType stateEventType);

    public class DatabaseException : Exception
    {
        public DatabaseException()
            : base()
        {
        }

        public DatabaseException(string message)
            : base(message)
        {
        }

        [SecuritySafeCritical]
        protected DatabaseException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public DatabaseException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

        public static bool IsUniqueKeyViolation(Exception ex)
        {
            bool result = false;
            if (ex is SqlException)
            {
                SqlException sqlEx = (SqlException)ex;
                result = ((sqlEx.Class == 14) && ((sqlEx.Number == 2601) || (sqlEx.Number == 2627)));
            }
            return result;
        }

        public static bool IsDeleteForeignKeyViolation(Exception ex)
        {
            bool result = false;
            if (ex is SqlException)
            {
                SqlException sqlEx = (SqlException)ex;
                result = ((sqlEx.Class == 16) && (sqlEx.Number == 547) && (!string.IsNullOrWhiteSpace(sqlEx.Message) && 
                          (sqlEx.Message.IndexOf("DELETE", StringComparison.InvariantCultureIgnoreCase) >= 0)));
            }
            return result;
        }
    }

    public class Transaction : IDisposable
    {
        private Database _database = null;

        public Transaction(Database database)
        {
            this._database = database;
            if (this._database._transaction != null)
            {
                throw new DatabaseException("Transaction error");
            }
            if (this._database.Connection == null)
            {
                this._database.Connect();
            }
            this._database._transaction = this._database.Connection.BeginTransaction(IsolationLevel.ReadCommitted);
        }

        public void Commit()
        {
            if (this._database._transaction != null)
            {
                try
                {
                    this._database._transaction.Commit();
                }
                catch
                {
                }
                this._database._transaction.Dispose();
                this._database._transaction = null;
            }
        }

        public void Rollback()
        {
            if (this._database._transaction != null)
            {
                try
                {
                    this._database._transaction.Rollback();
                }
                catch
                {
                }
                this._database._transaction.Dispose();
                this._database._transaction = null;
            }
        }

        public void Dispose()
        {
            Commit();
        }
    }

    public class Database : IDisposable
    {
        private SqlConnection _connection = null;
        private string _connectionString = null;
        private bool _connectionLost = false;
        internal protected SqlTransaction _transaction = null;

        public Database()
        {
            this._connectionString = ConfigurationManager.ConnectionStrings["ePlanner"].ConnectionString;
        }

        public event ConnectionStateChangedEventHandler ConnectionStateChanged;

        public static object ConvertValueToDB(object value)
        {
            object result = DBNull.Value;
            if (value != null)
            {
                result = value;
                if (result is string)
                {
                    if (string.IsNullOrEmpty((string)result))
                    {
                        result = DBNull.Value;
                    }
                }
            }
            return result;
        }

        public void Dispose()
        {
            Disconnect();
        }

        private void DoConnectionStateChanged(ConnectionStateEventType stateEventType)
        {
            if (this.ConnectionStateChanged != null)
            {
                this.ConnectionStateChanged(this, stateEventType);
            }
        }

        private void Connection_StateChange(object sender, StateChangeEventArgs e)
        {
            if ((e.OriginalState == ConnectionState.Broken) || (e.OriginalState == ConnectionState.Closed))
            {
                if (e.CurrentState == ConnectionState.Open)
                {
                    if (this._connectionLost)
                    {
                        this._connectionLost = false;
                        DoConnectionStateChanged(ConnectionStateEventType.Restore);
                    }
                    else
                    {
                        DoConnectionStateChanged(ConnectionStateEventType.Open);
                    }
                }
            }
            else
            {
                if ((e.CurrentState == ConnectionState.Broken) || (e.CurrentState == ConnectionState.Closed))
                {
                    DoConnectionStateChanged(ConnectionStateEventType.Closed);
                }
            }
        }

        private bool IsDatabaseLost()
        {
            bool result = true;
            try
            {
                using (SqlCommand command = new SqlCommand("select @@VERSION", this._connection, this._transaction))
                {
                    command.ExecuteScalar();
                    result = false;
                }
            }
            catch
            {
            }
            if (result)
            {
                DoConnectionStateChanged(ConnectionStateEventType.Lost);
            }
            return result;
        }

        public SqlConnection Connection
        {
            get
            {
                return this._connection;
            }
        }

        public string ConnectionString
        {
            get
            {
                return this._connectionString;
            }
            set
            {
                if (!string.Equals(this._connectionString, value, StringComparison.InvariantCultureIgnoreCase))
                {
                    this._connectionString = value;
                    Disconnect();
                }
            }
        }

        public bool Connect(bool canThrowException = true)
        {
            Exception error;
            bool result = Connect(out error);
            if (canThrowException && (error != null))
            {
                throw new DatabaseException("Unable to connect to database", error);
            }
            return result;
        }

        public bool Connect(out Exception error)
        {
            bool result = false;
            error = null;
            if (this._connection == null)
            {
                if (string.IsNullOrWhiteSpace(this._connectionString))
                {
                    error = new DatabaseException("Connection string not defined");
                }
                else
                {
                    this._connection = new SqlConnection(this._connectionString);
                    this._connection.StateChange += new StateChangeEventHandler(Connection_StateChange);
                }
            }
            if (this._connection != null)
            {
                if ((this._connection.State == System.Data.ConnectionState.Broken) || (this._connection.State == System.Data.ConnectionState.Closed))
                {
                    try
                    {
                        this._connection.Open();
                        result = true;
                    }
                    catch (Exception ex)
                    {
                        error = ex;
                    }
                }
                else
                {
                    result = true;
                }
            }
            return result;
        }

        public void Disconnect()
        {
            if (this._connection != null)
            {
                this._connection.Close();
                this._connection.Dispose();
                this._connection = null;
            }
        }

        private void ThrowDatabaseException(Exception ex, string customMessage)
        {
            if (IsDatabaseLost())
            {
                Disconnect();
                throw new DatabaseException("Database connection lost", ex);
            }
            else
            {
                if (!string.IsNullOrWhiteSpace(customMessage))
                {
                    throw new DatabaseException(customMessage, ex);
                }
                else
                {
                    throw ex;
                }
            }
        }

        public DataTable Select(string sql)
        {
            return Select(sql, null);
        }

        public DataTable Select(string sql, SqlParameter[] parameters)
        {
            DataTable result = null;
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(sql, this._connection, this._transaction))
                {
                    if ((parameters != null) && (parameters.Count() > 0))
                    {
                        command.Parameters.AddRange(parameters);
                    }
                    using (SqlDataAdapter adapter = new SqlDataAdapter(command))
                    {
                        result = new DataTable();
                        try
                        {
                            adapter.Fill(result);
                        }
                        catch (Exception ex)
                        {
                            result.Dispose();
                            result = null;
                            ThrowDatabaseException(ex, null);
                        }
                    }
                }
            }
            return result;
        }

        public Nullable<int> SelectInt(string sql)
        {
            return SelectInt(sql, null);
        }

        public Nullable<int> SelectInt(string sql, SqlParameter[] parameters)
        {
            Nullable<int> result = null;
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(sql, this._connection, this._transaction))
                {
                    if ((parameters != null) && (parameters.Count() > 0))
                    {
                        command.Parameters.AddRange(parameters);
                    }
                    try
                    {
                        object value = command.ExecuteScalar();
                        if ((value != null) && !Convert.IsDBNull(value))
                        {
                            result = Convert.ToInt32(value);
                        }
                    }
                    catch (Exception ex)
                    {
                        ThrowDatabaseException(ex, null);
                    }
                }
            }
            return result;
        }

        public Nullable<double> SelectFloat(string sql)
        {
            return SelectFloat(sql, null);
        }

        public Nullable<double> SelectFloat(string sql, SqlParameter[] parameters)
        {
            Nullable<double> result = null;
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(sql, this._connection, this._transaction))
                {
                    if ((parameters != null) && (parameters.Count() > 0))
                    {
                        command.Parameters.AddRange(parameters);
                    }
                    try
                    {
                        object value = command.ExecuteScalar();
                        if ((value != null) && !Convert.IsDBNull(value))
                        {
                            result = Convert.ToDouble(value);
                        }
                    }
                    catch (Exception ex)
                    {
                        ThrowDatabaseException(ex, null);
                    }
                }
            }
            return result;
        }

        public string SelectString(string sql)
        {
            return SelectString(sql, null);
        }

        public string SelectString(string sql, SqlParameter[] parameters)
        {
            string result = null;
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(sql, this._connection, this._transaction))
                {
                    if ((parameters != null) && (parameters.Count() > 0))
                    {
                        command.Parameters.AddRange(parameters);
                    }
                    try
                    {
                        object value = command.ExecuteScalar();
                        if ((value != null) && !Convert.IsDBNull(value))
                        {
                            result = Convert.ToString(value);
                        }
                    }
                    catch (Exception ex)
                    {
                        ThrowDatabaseException(ex, null);
                    }
                }
            }
            return result;
        }

        public Nullable<DateTime> SelectDateTime(string sql)
        {
            return SelectDateTime(sql, null);
        }

        public Nullable<DateTime> SelectDateTime(string sql, SqlParameter[] parameters)
        {
            Nullable<DateTime> result = null;
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(sql, this._connection, this._transaction))
                {
                    if ((parameters != null) && (parameters.Count() > 0))
                    {
                        command.Parameters.AddRange(parameters);
                    }
                    try
                    {
                        object value = command.ExecuteScalar();
                        if ((value != null) && !Convert.IsDBNull(value))
                        {
                            result = Convert.ToDateTime(value);
                        }
                    }
                    catch (Exception ex)
                    {
                        ThrowDatabaseException(ex, null);
                    }
                }
            }
            return result;
        }

        public int Update(string sql)
        {
            return Update(sql, null);
        }

        public int Update(string sql, SqlParameter[] parameters)
        {
            int result = -1;
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(sql, this._connection, this._transaction))
                {
                    if ((parameters != null) && (parameters.Count() > 0))
                    {
                        command.Parameters.AddRange(parameters);
                    }
                    try
                    {
                        result = command.ExecuteNonQuery();
                    }
                    catch (Exception ex)
                    {
                        ThrowDatabaseException(ex, null);
                    }
                }
            }
            return result;
        }

        public int Insert(string sql)
        {
            return Insert(sql, null);
        }

        public int Insert(string sql, SqlParameter[] parameters)
        {
            return Update(sql, parameters);
        }

        public void ExecuteProcedure(string name)
        {
            if (Connect())
            {
                using (SqlCommand command = new SqlCommand(name, this._connection, this._transaction))
                {
                    try
                    {
                        command.CommandType = CommandType.StoredProcedure;
                        command.ExecuteNonQuery();
                    }
                    catch (Exception ex)
                    {
                        ThrowDatabaseException(ex, null);
                    }
                }
            }
        }

        public static DateTime GetNowTime()
        {
            DateTime result = DateTime.Now;
            try
            {
                using (Database database = new Database())
                {
                    string sql = "select getdate()";
                    DateTime? dbTime = database.SelectDateTime(sql);
                    if (dbTime.HasValue)
                    {
                        result = dbTime.Value;
                    }
                }
            }
            catch
            {
            }
            return result;
        }

        public static bool TestConnection(out string error)
        {
            bool result = false;
            error = null;
            try
            {
                using (Database database = new Database())
                {
                    Exception ex;
                    result = database.Connect(out ex);
                    if (ex != null)
                    {
                        error = ex.ToString();
                    }
                }
            }
            catch (Exception ex)
            {
                error = ex.ToString();
            }
            return result;
        }

        public static int ConvertToInt(object value, int defaultValue = 0)
        {
            int result = defaultValue;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                result = Convert.ToInt32(value);
            }
            return result;
        }

        public static double ConvertToFloat(object value, double defaultValue = 0.0)
        {
            double result = defaultValue;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                result = Convert.ToDouble(value);
            }
            return result;
        }

        public static decimal ConvertToDecimal(object value, decimal defaultValue = 0)
        {
            decimal result = defaultValue;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                result = Convert.ToDecimal(value);
            }
            return result;
        }

        public static string ConvertToStr(object value, string defaultValue = null)
        {
            string result = defaultValue;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                if (value is byte[])
                {
                    byte[] buffer = (byte[])value;
                    if ((buffer != null) && (buffer.Length > 0))
                    {
                        result = Encoding.UTF8.GetString(buffer);
                    }
                }
                else
                {
                    result = Convert.ToString(value);
                }
            }
            return result;
        }

        public static byte[] ConvertToByteArray(object value)
        {
            byte[] result = null;
            if ((value != null) && !Convert.IsDBNull(value) && (value is byte[]))
            {
                result = (byte[])value;
            }
            return result;
        }

        public static bool ConvertToBool(object value, bool defaultValue = false)
        {
            bool result = defaultValue;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                if (value is bool)
                {
                    result = (bool)value;
                }
                else
                {
                    string s = Convert.ToString(value).ToUpper().Trim();
                    result = (s == "A") || (s == "Y") || (s == "1");
                }
            }
            return result;
        }

        public static Nullable<DateTime> ConvertToNullableDateTime(object value)
        {
            Nullable<DateTime> result = null;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                result = Convert.ToDateTime(value);
            }
            return result;
        }


        public static DateTime ConvertToDateTime(object value)
        {
            DateTime result = DateTime.MinValue;
            if ((value != null) && !Convert.IsDBNull(value))
            {
                result = Convert.ToDateTime(value);
            }
            return result;
        }

        public static bool IsNull(object value)
        {
            return (value == null) || Convert.IsDBNull(value);
        }
    }
    /*
    public static class GridViewExtensions
    {
        public static T GetSelectedObject<T>(GridView view, int rowHandle = -1) where T : class
        {
            int[] rowIDs;
            if (rowHandle >= 0)
            {
                rowIDs = new int[] { rowHandle };
            }
            else
            {
                rowIDs = view.GetSelectedRows();
            }
            if (rowIDs.Length > 0)
            {
                int rowID = rowIDs[0];
                if (rowID != -1)
                {
                    return (T)view.GetRow(rowID);
                }
            }
            return default(T);
        }

        public static List<DataRow> GetSelectedDataRows(GridView view)
        {
            List<DataRow> result = new List<DataRow>();
            int[] rowIDs = view.GetSelectedRows();
            foreach (int id in rowIDs)
            {
                result.Add(view.GetDataRow(id));
            }
            return result;
        }

        public static DataRow GetSelectedDataRow(GridView view)
        {
            DataRow result = null;
            List<DataRow> dataRows = GetSelectedDataRows(view);
            if (dataRows.Count > 0)
            {
                result = dataRows[0];
            }
            return result;
        }

        public static DataRow GetTopVisibleDataRow(GridView view)
        {
            DataRow result = null;
            if (view.TopRowIndex >= 0)
            {
                result = view.GetDataRow(view.TopRowIndex);
            }
            return result;
        }

        public static DataRow GetFocusedDataRow(GridView view)
        {
            DataRow result = null;
            int handle = view.FocusedRowHandle;
            if (handle >= 0)
            {
                result = view.GetDataRow(handle);
            }
            return result;
        }
       
    }
     */
}
